/* $Id: algo_lib.c,v 1.2 2002/01/24 23:21:11 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#include <stdlib.h>
#include <string.h>
#include "algo_int.h"

struct algorithm_context_st
{
  const SYM_METHOD *sym;
  const MODE_METHOD *mode;
  ALGO_CTX *sym_data;		/* The context used by the symmetric method */
  ALGO_CTX *mode_data;		/* The context used by the mode method */
};

ALGO_CTX *SYM_new(const SYM_METHOD *sym, const MODE_METHOD *mode)
{
  size_t amount;
  ALGO_CTX *ptr;

  if (!sym)
    return 0;

  amount = sizeof(ALGO_CTX) + sym->algo_ctx_size()
    + (mode ? mode->algo_ctx_size() : 0);
  ptr = malloc(amount);
  ptr->sym = sym;
  ptr->sym_data = (ALGO_CTX *)((char *)ptr + sizeof(ALGO_CTX));
  ptr->mode = mode;
  ptr->mode_data =
    mode ? (ALGO_CTX *)((char *)ptr + sizeof(ALGO_CTX) + sym->algo_ctx_size())
    : 0;
  memset(ptr->sym_data, 0, sym->algo_ctx_size()
	 + (mode ? mode->algo_ctx_size() : 0));

  return ptr;
}

int SYM_init(ALGO_CTX *ctx, const unsigned char *inkey, size_t keylen,
	     const unsigned char *ivec, size_t iveclen)
{
  int status;

  if (ivec && iveclen && iveclen != SYM_get_block_bits(ctx->sym)/8)
    return ALGO_E_INVALID_IVEC_SIZE;
  if (!ivec)
    iveclen = SYM_get_block_bits(ctx->sym)/8;

  status = ctx->sym->init(ctx->sym_data, inkey, keylen);
  if (ctx->mode && status == ALGO_E_OK)
    status = ctx->mode->init(ctx->mode_data, ivec, iveclen);

  return status;
}

int SYM_encrypt(ALGO_CTX *ctx,
		const unsigned char *input, unsigned char *output,
		size_t length)
{
  if (ctx->mode)
    return ctx->mode->enc_dec(ctx->mode_data, input, output, length,
			      OPENSSL_ENCRYPT,
			      ctx->sym->enc_dec, ctx->sym_data);
  return ctx->sym->enc_dec(ctx->sym_data, input, output, length,
			   OPENSSL_ENCRYPT);
}

int SYM_decrypt(ALGO_CTX *ctx,
		const unsigned char *input, unsigned char *output,
		size_t length)
{
  if (ctx->mode)
    return ctx->mode->enc_dec(ctx->mode_data, input, output, length,
			      OPENSSL_DECRYPT,
			      ctx->sym->enc_dec, ctx->sym_data);
  return ctx->sym->enc_dec(ctx->sym_data, input, output, length,
			   OPENSSL_DECRYPT);
}

void SYM_free(ALGO_CTX *ctx)
{
  ctx->sym->clear(ctx->sym_data);
  if (ctx->mode)
    ctx->mode->clear(ctx->mode_data);
  free(ctx);
}

size_t SYM_get_key_bits(const SYM_METHOD *sym)
{
  return sym->key_bits;
}

size_t SYM_get_block_bits(const SYM_METHOD *sym)
{
  return sym->block_bits;
}

const char *SYM_get_name(const SYM_METHOD *sym)
{
  return sym->name;
}

const SYM_METHOD *SYM_get_method(const ALGO_CTX *ctx)
{
  return ctx->sym;
}

int MODE_cpy_ivec(ALGO_CTX *ctx, unsigned char *ivec, size_t ivecsize,
		  size_t *iveclen)
{
  if (ctx->mode)
    return ctx->mode->cpy_ivec(ctx->mode_data, ivec, ivecsize, iveclen);
  return ALGO_E_NO_IVEC;
}

const char *MODE_get_name(const MODE_METHOD *mode)
{
  return mode->name;
}

const MODE_METHOD *MODE_get_method(const ALGO_CTX *ctx)
{
  return ctx->mode;
}

