/* $Id: algorithms.h,v 1.2 2002/01/24 23:21:11 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#ifndef HEADER_ALGORITHMS_H
#define HEADER_ALGORITHMS_H

#include <sys/types.h>

#ifdef _cplusplus
extern "C"
{
#endif

  /* Some functions take a parameter saying whether the input should
     be encrypted or decrypted.  The folowing values are accepted. */
#define OPENSSL_DECRYPT 0
#define OPENSSL_ENCRYPT 1

  /* A generic type for hidden info used by the algorithm functions and
     methods.  It may mean different things in different places */
  typedef struct algorithm_context_st ALGO_CTX;

  /****************************************************************************
   * Symmetric algorithms and algorithm modes
   **/

  /* The method structures. */
  typedef struct symmetric_method_st SYM_METHOD;
  typedef struct symmetric_mode_method_st MODE_METHOD;

  /* Functions to use existing algorithms */
  ALGO_CTX *SYM_new(const SYM_METHOD *sym, const MODE_METHOD *mode);
  int SYM_init(ALGO_CTX *ctx, const unsigned char *inkey, size_t keylen,
	       const unsigned char *ivec, size_t iveclen);
  int SYM_encrypt(ALGO_CTX *ctx,
		  const unsigned char *input, unsigned char *output,
		  size_t length);
  int SYM_decrypt(ALGO_CTX *ctx,
		  const unsigned char *input, unsigned char *output,
		  size_t length);
  void SYM_free(ALGO_CTX *ctx);

  size_t SYM_get_key_bits(const SYM_METHOD *meth);
  size_t SYM_get_block_bits(const SYM_METHOD *meth);
  const char *SYM_get_name(const SYM_METHOD *meth);
  const SYM_METHOD *SYM_get_method(const ALGO_CTX *ctx);

  int MODE_cpy_ivec(ALGO_CTX *ctx, unsigned char *ivec, size_t ivecsize,
		    size_t *iveclen);
  const char *MODE_get_name(const MODE_METHOD *meth);
  const MODE_METHOD *MODE_get_method(const ALGO_CTX *ctx);

  /* Functions to create new algorithms.
     You're supposed to create a method structure with SYM_create(), alter
     the values with the SYM_set_*() functions and finally lock all those
     values into the structure with SYM_lock_values().  When the algorithm
     isn't needed any more, it can be discarded with SYM_destroy(). */
  SYM_METHOD *SYM_create(void);
  int SYM_set_name(SYM_METHOD *meth, const char *name);
  int SYM_set_nid(SYM_METHOD *meth, int nid);
  int SYM_set_key_bits(SYM_METHOD *meth, size_t keylen);
  int SYM_set_block_bits(SYM_METHOD *meth, size_t blocklen);
  int SYM_set_init_fn(SYM_METHOD *meth,
		      int (*fn)(ALGO_CTX *, const unsigned char *, size_t));
  int SYM_set_enc_dec_fn(SYM_METHOD *meth,
			 int (*fn)(ALGO_CTX *,
				   const unsigned char *, unsigned char *, size_t,
				   int));
  int SYM_set_alloc_fn(SYM_METHOD *meth, int (*fn)(void));
  int SYM_set_free_fn(SYM_METHOD *meth, int (*fn)(ALGO_CTX *));
  int SYM_lock_values(SYM_METHOD *meth);
  void SYM_destroy(SYM_METHOD *meth);

  /* Functions to create new modes.
     You're supposed to create a method structure with MODE_create(), alter
     the values with the MODE_set_*() functions and finally lock all those
     values into the structure with MODE_lock_values().  When the algorithm
     isn't needed any more, it can be discarded with MODE_destroy(). */
  MODE_METHOD *MODE_create(void);
  int MODE_set_name(MODE_METHOD *meth, const char *name);
  int MODE_set_nid(MODE_METHOD *meth, int nid);
  int MODE_set_init_fn(MODE_METHOD *meth, int (*fn)(ALGO_CTX *, const unsigned char *));
  int MODE_set_enc_dec_fn(MODE_METHOD *meth,
			  int (*fn)(ALGO_CTX *,
				    const char *, char *, size_t,
				    int));
  int MODE_set_alloc_fn(MODE_METHOD *meth, int (*fn)(void));
  int MODE_set_free_fn(MODE_METHOD *meth, int (*fn)(ALGO_CTX *));
  int MODE_set_get_ivec(MODE_METHOD *meth, int (*fn)(ALGO_CTX *,
						     unsigned char *, size_t,
						     size_t *));
  int MODE_lock_values(MODE_METHOD *meth);
  void MODE_destroy(MODE_METHOD *meth);

  /* All functions returning int return a status code: */
#define ALGO_E_OK			0
#define ALGO_E_NULL_ARGUMENTS		1
#define ALGO_E_INVALID_KEY_SIZE		2
#define ALGO_E_INVALID_BLOCK_SIZE	3
#define ALGO_E_INVALID_IVEC_SIZE	4
#define ALGO_E_TOO_LARGE_BLOCK		5
#define ALGO_E_NO_MEMORY		6
#define ALGO_E_NO_IVEC			7
#define ALGO_E_NOT_EXACT_BLOCKS		8
#define ALGO_E_INCORRECT_PARITY		9
#define ALGO_E_WEAK_KEY			10

  /* Built-in methods: */
  const SYM_METHOD *AES(void);
  const SYM_METHOD *BLOWFISH(void);
  const MODE_METHOD *ECB(void);
  const MODE_METHOD *CBC(void);
  const MODE_METHOD *OFB(void);
  const MODE_METHOD *CFB1(void);
  const MODE_METHOD *CFB8(void);
  const MODE_METHOD *CFB64(void);
  const MODE_METHOD *CFB128(void);

#ifdef _cplusplus
}
#endif

#endif
